//*****************************************************
//
// Exemplo3D.cpp 
// Um programa OpenGL simples que abre uma janela GLUT 
// e desenha um cubo para exemplificar a visualizao
// de objetos 3D utilizando a projeo perspectiva.
//
// Marcelo Cohen e Isabel H. Manssour
// Este cdigo acompanha o livro 
// "OpenGL - Uma Abordagem Prtica e Objetiva"
// 
//*****************************************************

#include <stdlib.h>
#include <GL/glut.h>

GLfloat fAspect;
GLfloat angulo;

void alteraAngulo(int incrementoAngulo) {
  angulo += incrementoAngulo;
  
  if (angulo >= 360.0f) {
     angulo = 0.0f;
  }
  
  // Invoca novamente a rotina callback do timer
  glutTimerFunc(25, alteraAngulo, 1);

  // Resedenha a cena
  glutPostRedisplay();

}

// Funo callback de redesenho da janela de visualizao
void Desenha(void)
{
	// Limpa a janela de visualizao com a cor  
	// de fundo definida previamente
	glClear(GL_COLOR_BUFFER_BIT);

	// Altera a cor do desenho para preto
	glColor3f(0.0f, 0.0f, 0.0f);

	// Salva a matriz corrente do modelo
        glPushMatrix();
        
        // Rotaciona em torno do eixo Y
        glRotatef(angulo, 0, 1, 0);
        
        // Funo da GLUT para fazer o desenho de um cubo 
	// com a cor corrente
	glutWireCube(50);
        
        // Restaura a matriz do modelo (antes da rotacao)
        glPopMatrix();

	// Executa os comandos OpenGL
	glFlush();
}

// Funo usada para especificar o volume de visualizao
void EspecificaParametrosVisualizacao(void)
{
	// Especifica sistema de coordenadas de projeo
	glMatrixMode(GL_PROJECTION);
	// Inicializa sistema de coordenadas de projeo
	glLoadIdentity();

	// Especifica a projeo perspectiva(angulo,aspecto,zMin,zMax)
	gluPerspective(60,fAspect,0.5,500);

	// Especifica sistema de coordenadas do modelo
	glMatrixMode(GL_MODELVIEW);
	// Inicializa sistema de coordenadas do modelo
	glLoadIdentity();

	// Especifica posio do observador e do alvo
	gluLookAt(40,60,100, 0,0,0, 0,1,0);
}

// Funo callback chamada quando o tamanho da janela  alterado 
void AlteraTamanhoJanela(GLsizei w, GLsizei h)
{
	// Para previnir uma diviso por zero
	if ( h == 0 ) h = 1;

	// Especifica as dimenses da viewport
	glViewport(0, 0, w, h);
 
	// Calcula a correo de aspecto
	fAspect = (GLfloat)w/(GLfloat)h;

	EspecificaParametrosVisualizacao();
}

// Funo callback chamada para gerenciar eventos de teclas
void Teclado (unsigned char key, int x, int y)
{
	if (key == 27)
		exit(0);
}

// Funo responsvel por inicializar parmetros e variveis
void Inicializa (void)
{   
	// Define a cor de fundo da janela de visualizao como branca
	glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
	glLineWidth(2.0);
}

// Programa Principal 
int main(int argc, char *argv[])
{

        glutInit(&argc, argv);	

        // Define do modo de operao da GLUT
	glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB); 

	// Especifica a posio inicial da janela GLUT
	glutInitWindowPosition(5,5); 

	// Especifica o tamanho inicial em pixels da janela GLUT
	glutInitWindowSize(450,450); 
 
	// Cria a janela passando como argumento o ttulo da mesma
	glutCreateWindow("Desenho de um cubo");

	// Registra a funo callback de redesenho da janela de visualizao
	glutDisplayFunc(Desenha);

	// Registra a funo callback de redimensionamento da janela de visualizao
	glutReshapeFunc(AlteraTamanhoJanela);

	// Registra a funo callback para tratamento das teclas ASCII
	glutKeyboardFunc (Teclado);

	// Chama a funo responsvel por fazer as inicializaes 
	Inicializa();

        glutTimerFunc(50, alteraAngulo, 1);
 
	// Inicia o processamento e aguarda interaes do usurio
	glutMainLoop();

	return 0;
}
