//*****************************************************
//
// ExemploTextura2D.cpp 
// Um programa OpenGL simples que abre uma janela GLUT 
// e desenha um plano com uma textura 2D
//
// Marcelo Cohen e Isabel H. Manssour
// Este cdigo acompanha o livro 
// "OpenGL - Uma Abordagem Prtica e Objetiva"
// 
//*****************************************************

#include "bibutil.h"
#include <iostream>
using namespace std;


// Variveis para controles de navegao
GLfloat angle, fAspect;
GLfloat rotX, rotY, rotX_ini, rotY_ini;
GLfloat obsX, obsY, obsZ, obsX_ini, obsY_ini, obsZ_ini;
int x_ini,y_ini,bot;

//
const GLfloat inicio = 0.0f;
const GLfloat fim = 1.0f;

// Define uma textura 2D
TEX *textura;

// Define parmetros de iluminao
GLfloat luzAmb[4] = { 0.3, 0.3, 0.3, 1 };	// luz ambiente
GLfloat luzDif[4] = { 1.0, 1.0, 1.0, 1 };	// luz difusa
GLfloat posLuz[4] = { 0, 0, 10, 1 };		// posio da fonte de luz

// Define modo inicial como GL_MODULATE
GLint modo = GL_MODULATE;

// Funo callback de redesenho da janela de visualizao
void Desenha(void)
{
	// Limpa a janela de visualizao com a cor  
	// de fundo definida previamente
	glClear(GL_COLOR_BUFFER_BIT);

	glEnable(GL_TEXTURE_2D);
	
	glNormal3f(0,0,1);
	glBegin(GL_QUADS);
		glTexCoord2f(0.0f,0.0f);
        glVertex3f(-1,-1,0);
		glTexCoord2f(1.0,0);
		glVertex3f( 1,-1,0);
		glTexCoord2f(1.0,1.0);
		glVertex3f( 1, 1,0);
		glTexCoord2f(0,1.0);
		glVertex3f(-1, 1,0);
	glEnd();

	glDisable(GL_TEXTURE_2D);

    glPushMatrix();
    glTranslatef(3.0f, 0.0f, 0.0f);
	glBegin(GL_QUADS);
		glTexCoord2f(0.0f,0.0f);
        glVertex3f(-1,-1,0);
		glTexCoord2f(1.0,0);
		glVertex3f( 1,-1,0);
		glTexCoord2f(1.0,1.0);
		glVertex3f( 1, 1,0);
		glTexCoord2f(0,1.0);
		glVertex3f(-1, 1,0);
	glEnd();
    glPopMatrix();
    
	// Executa os comandos OpenGL
	glutSwapBuffers();
}

// Funo usada para especificar a posio do observador virtual
void PosicionaObservador(void)
{
	// Especifica sistema de coordenadas do modelo
	glMatrixMode(GL_MODELVIEW);
	// Inicializa sistema de coordenadas do modelo
	glLoadIdentity();
	// Seta posio da luz
	glLightfv( GL_LIGHT0, GL_POSITION, posLuz ); 
	// Posiciona e orienta o observador
	glTranslatef(-obsX,-obsY,-obsZ);
	glRotatef(rotX,1,0,0);
	glRotatef(rotY,0,1,0);
}

// Funo callback para eventos de teclado
void Teclado(unsigned char key, int x, int y)
{
	// Trata as teclas ESC e 'm'
	switch(key)
	{
		case 27:	LiberaMateriais();
					exit(1);
					break;
		case 'm':	modo = (modo == GL_REPLACE ? GL_MODULATE: GL_REPLACE);
					break;	
	}

	// Ajusta o modo de aplicao da textura
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, modo);

	glutPostRedisplay();
}

// Funo usada para especificar o volume de visualizao
void EspecificaParametrosVisualizacao(void)
{
	// Especifica sistema de coordenadas de projeo
	glMatrixMode(GL_PROJECTION);
	// Inicializa sistema de coordenadas de projeo
	glLoadIdentity();

	// Especifica a projeo perspectiva 
	// (angulo, aspecto, zMin, zMax)
	gluPerspective(90,fAspect,0.1,500);

	// Especifica sistema de coordenadas do modelo
	glMatrixMode(GL_MODELVIEW);
	PosicionaObservador();
}

// Funo callback chamada quando o tamanho da janela  alterado 
void AlteraTamanhoJanela(GLsizei w, GLsizei h)
{
	// Para previnir uma diviso por zero
	if ( h == 0 ) h = 1;

	// Especifica as dimenses da viewport
	glViewport(0, 0, w, h);
 
	// Calcula a correo de aspecto
	fAspect = (GLfloat)w/(GLfloat)h;

	EspecificaParametrosVisualizacao();
}

// Funo callback para eventos de botes do mouse
void GerenciaMouse(int button, int state, int x, int y)
{
	if(state==GLUT_DOWN)
	{
		// Salva os parmetros atuais
		x_ini = x;
		y_ini = y;
		obsX_ini = obsX;
		obsY_ini = obsY;
		obsZ_ini = obsZ;
		rotX_ini = rotX;
		rotY_ini = rotY;
		bot = button;
	}
	else bot = -1;
}

// Funo callback para eventos de movimento do mouse
#define SENS_ROT	5.0
#define SENS_OBS	100.0
#define SENS_TRANSL	100.0
void GerenciaMovim(int x, int y)
{
	// Boto esquerdo ?
	if(bot==GLUT_LEFT_BUTTON)
	{
		// Calcula diferenas
		int deltax = x_ini - x;
		int deltay = y_ini - y;
		// E modifica ngulos
		rotY = rotY_ini - deltax/SENS_ROT;
		rotX = rotX_ini - deltay/SENS_ROT;
	}
	// Boto direito ?
	else if(bot==GLUT_RIGHT_BUTTON)
	{
		// Calcula diferena
		int deltaz = y_ini - y;
		// E modifica distncia do observador
		obsZ = obsZ_ini + deltaz/SENS_OBS;
	}
	// Boto do meio ?
	else if(bot==GLUT_MIDDLE_BUTTON)
	{
		// Calcula diferenas
		int deltax = x_ini - x;
		int deltay = y_ini - y;
		// E modifica posies
		obsX = obsX_ini + deltax/SENS_TRANSL;
		obsY = obsY_ini - deltay/SENS_TRANSL;
	}
	PosicionaObservador();
	glutPostRedisplay();
}

// Funo responsvel por inicializar parmetros e variveis
void Inicializa (void)
{   
	// Define a cor de fundo da janela de visualizao como branca
	glClearColor(1.0f, 1.0f, 1.0f, 1.0f);

	// Carrega a textura e envia para OpenGL
	textura = CarregaTextura("azulejo.jpg",false);

	// Associa a textura aos comandos seguintes
	glBindTexture(GL_TEXTURE_2D, textura->texid);

	// Seleciona o modo de aplicao da textura
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	// Iluminao
	glLightfv( GL_LIGHT0, GL_AMBIENT,  luzAmb ); 
	glLightfv( GL_LIGHT0, GL_DIFFUSE,  luzDif );

	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHTING);

	// Inicializa as variveis usadas para alterar a posio do 
	// observador virtual
	rotX = 0;
	rotY = 0;
	obsX = obsY = 0;
	obsZ = 1.5;
}

// Programa Principal 
int main(void)
{
	// Define do modo de operao da GLUT
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB); 
 
	// Especifica o tamanho inicial em pixels da janela GLUT
	glutInitWindowSize(450,450); 
 
	// Cria a janela passando como argumento o ttulo da mesma
	glutCreateWindow("Texturas 2D");
 
	// Registra a funo callback de redesenho da janela de visualizao
	glutDisplayFunc(Desenha);
  
	// Registra a funo callback de teclado
	glutKeyboardFunc(Teclado);

	// Registra a funo callback para eventos de botes do mouse	
	glutMouseFunc(GerenciaMouse);
	 
	// Registra a funo callback para eventos de movimento do mouse	
	glutMotionFunc(GerenciaMovim);

	// Registra a funo callback de redimensionamento da janela de visualizao
	glutReshapeFunc(AlteraTamanhoJanela);
    
	// Chama a funo responsvel por fazer as inicializaes 
	Inicializa();
 
	// Inicia o processamento e aguarda interaes do usurio
	glutMainLoop();
 
	return 0;
}
