//*****************************************************
//
// Exemplo3D.cpp 
// Um programa OpenGL simples que abre uma janela GLUT 
// e desenha um cubo para exemplificar a visualizao
// de objetos 3D utilizando a projeo perspectiva.
//
// Marcelo Cohen e Isabel H. Manssour
// Este cdigo acompanha o livro 
// "OpenGL - Uma Abordagem Prtica e Objetiva"
// 
//*****************************************************

#include <stdlib.h>
#include <GL/glut.h>
//Biblioteca auxiliar para trabalhar com texturas
#include "bibutil.h"

// Variveis para controles de navegao
GLfloat fAspect;

// Define uma textura 2D
TEX *textura;

// Define valor final da coordenada de textura
float final = 1.0;

void desenhaCubo() {


	glEnable(GL_TEXTURE_2D);

    glNormal3f(0,0,1);
	
	glBegin ( GL_QUADS );
               
		// Front Face
        glTexCoord2f(0,0);
		glVertex3f(-1.0f, -1.0f,  1.0f);
        glTexCoord2f(final,0);
		glVertex3f( 1.0f, -1.0f,  1.0f);
        glTexCoord2f(final,final);
		glVertex3f( 1.0f,  1.0f,  1.0f);
        glTexCoord2f(0,final);
		glVertex3f(-1.0f,  1.0f,  1.0f);
              
		// Back Face
        glColor3f(0.0f, 0.0f, 0.0f);
		glVertex3f(-1.0f, -1.0f, -1.0f);
        glVertex3f(-1.0f,  1.0f, -1.0f);
		glVertex3f( 1.0f,  1.0f, -1.0f);
		glVertex3f( 1.0f, -1.0f, -1.0f);

		// Top Face
		glVertex3f(-1.0f,  1.0f, -1.0f);
		glVertex3f(-1.0f,  1.0f,  1.0f);
		glVertex3f( 1.0f,  1.0f,  1.0f);
		glVertex3f( 1.0f,  1.0f, -1.0f);

		// Bottom Face
		glVertex3f(-1.0f, -1.0f, -1.0f);
		glVertex3f( 1.0f, -1.0f, -1.0f);
		glVertex3f( 1.0f, -1.0f,  1.0f);
		glVertex3f(-1.0f, -1.0f,  1.0f);

		// Right face
		glVertex3f( 1.0f, -1.0f, -1.0f);
		glVertex3f( 1.0f,  1.0f, -1.0f);
		glVertex3f( 1.0f,  1.0f,  1.0f);
		glVertex3f( 1.0f, -1.0f,  1.0f);

		// Left Face
		glVertex3f(-1.0f, -1.0f, -1.0f);
		glVertex3f(-1.0f, -1.0f,  1.0f);
		glVertex3f(-1.0f,  1.0f,  1.0f);
		glVertex3f(-1.0f,  1.0f, -1.0f);

	glEnd();
	
	glDisable(GL_TEXTURE_2D);
	
	
}

// Funo callback de redesenho da janela de visualizao
void Desenha(void)
{
	// Limpa a janela de visualizao com a cor  
	// de fundo definida previamente
	glClear(GL_COLOR_BUFFER_BIT);

    // Limpa o buffer de profundidade
    glClear(GL_DEPTH_BUFFER_BIT);

    // Funo para fazer o desenho de um cubo 
    desenhaCubo();
        
	// Troca os buffers de desenho
	glutSwapBuffers();
}

// Funo usada para especificar o volume de visualizao
void EspecificaParametrosVisualizacao(void)
{
	// Especifica sistema de coordenadas de projeo
	glMatrixMode(GL_PROJECTION);
	// Inicializa sistema de coordenadas de projeo
	glLoadIdentity();

	// Especifica a projeo perspectiva(angulo,aspecto,zMin,zMax)
	gluPerspective(60, fAspect, 0.5, 500);

	// Especifica sistema de coordenadas do modelo
	glMatrixMode(GL_MODELVIEW);
	// Inicializa sistema de coordenadas do modelo
	glLoadIdentity();

	// Especifica posio do observador e do alvo
	gluLookAt(3, 3, 3, 0,0,0, 0,1,0);
}

// Funo callback chamada quando o tamanho da janela  alterado 
void AlteraTamanhoJanela(GLsizei w, GLsizei h)
{
	// Para previnir uma diviso por zero
	if ( h == 0 ) h = 1;

	// Especifica as dimenses da viewport
	glViewport(0, 0, w, h);
 
	// Calcula a correo de aspecto
	fAspect = (GLfloat)w/(GLfloat)h;

	EspecificaParametrosVisualizacao();
}

// Funo responsvel por inicializar parmetros e variveis
void Inicializa (void)
{   
    glEnable(GL_DEPTH_TEST);

	// Define a cor de fundo da janela de visualizao como branca
	glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
	glLineWidth(2.0);
	fAspect = 1.0f;
	
	// Carrega a textura e envia para OpenGL
	textura = CarregaTextura("azulejo.jpg",false);

	// Associa a textura aos comandos seguintes
	glBindTexture(GL_TEXTURE_2D, textura->texid);

	// Seleciona o modo de aplicao da textura
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);	
}




// Programa Principal 
int main(int argc, char *argv[])
{

    glutInit(&argc, argv);	

    // Define do modo de operao da GLUT
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH); 

	// Especifica a posio inicial da janela GLUT
	glutInitWindowPosition(5,5); 

	// Especifica o tamanho inicial em pixels da janela GLUT
	glutInitWindowSize(450,450); 
 
	// Cria a janela passando como argumento o ttulo da mesma
	glutCreateWindow("Desenho de um cubo");

	// Registra a funo callback de redesenho da janela de visualizao
	glutDisplayFunc(Desenha);

	// Registra a funo callback de redimensionamento da janela de visualizao
	glutReshapeFunc(AlteraTamanhoJanela);

	// Chama a funo responsvel por fazer as inicializaes 
	Inicializa();

 	// Inicia o processamento e aguarda interaes do usurio
	glutMainLoop();

	return 0;
}
