#include <stdio.h>
#include <GL\glut.h>
#include "tgaload.h"

#define MAX_NO_TEXTURES 1
#define CUBE_TEXTURE 0

GLuint texture_id[MAX_NO_TEXTURES];

float xrot;
float yrot;
float zrot;
float ratio;

void initTexture (void)
{
	glEnable(GL_TEXTURE_2D);

	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

	glGenTextures(MAX_NO_TEXTURES, texture_id);
									
	glBindTexture(GL_TEXTURE_2D, texture_id[CUBE_TEXTURE]);

	image_t temp_image;
	tgaLoad("icmc.tga", &temp_image, TGA_FREE | TGA_LOW_QUALITY);
}

void init(void)
{
	glClearColor(0.0f, 0.0f, 0.25f, 1.0f);
	glEnable(GL_CULL_FACE);
}

void EspecificaParametrosDeVisualizacao()
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	gluPerspective(80, ratio, 1, 200);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	gluLookAt(0, 0, 10, 0, 0, 10, 0.0f, 1.0f, 0.0f);
}

void reshape(int w, int h)
{
	if (h == 0) h = 1;

	glViewport(0, 0, w, h);
	
	ratio = 1.0f * w / h;
	
	EspecificaParametrosDeVisualizacao();
}

void display(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	glLoadIdentity();
	glPushMatrix();
	
	glTranslatef(0.0, 0.0, -5.0);
	glRotatef(xrot, 1.0, 0.0, 0.0);
	glRotatef(yrot, 0.0, 1.0, 0.0);
	glRotatef(zrot, 0.0, 0.0, 1.0);

	glBindTexture(GL_TEXTURE_2D, texture_id[0]);
	
	glBegin ( GL_QUADS );
		// Front Face
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(-1.0f, -1.0f, 1.0f);
		
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f( 1.0f, -1.0f, 1.0f);
		
		glTexCoord2f(1.0f, 1.0f);
		glVertex3f( 1.0f, 1.0f, 1.0f);
		
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f(-1.0f, 1.0f, 1.0f);
		
		// Back Face
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f(-1.0f, -1.0f, -1.0f);
		
		glTexCoord2f(1.0f, 1.0f);
		glVertex3f(-1.0f,  1.0f, -1.0f);

		glTexCoord2f(0.0f, 1.0f);
		glVertex3f(1.0f,  1.0f, -1.0f);
		
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(1.0f, -1.0f, -1.0f);
		
		// Top Face
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f(-1.0f,  1.0f, -1.0f);
		
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(-1.0f,  1.0f, 1.0f);
		
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f(1.0f, 1.0f, 1.0f);
		
		glTexCoord2f(1.0f, 1.0f);
		glVertex3f(1.0f, 1.0f, -1.0f);
		
		// Bottom Face
		glTexCoord2f(1.0f, 1.0f);
		glVertex3f(-1.0f, -1.0f, -1.0f);

		glTexCoord2f(0.0f, 1.0f);
		glVertex3f(1.0f, -1.0f, -1.0f);

		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(1.0f, -1.0f, 1.0f);

		glTexCoord2f(1.0f, 0.0f);
		glVertex3f(-1.0f, -1.0f, 1.0f);
		
		// Right face
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f(1.0f, -1.0f, -1.0f);

		glTexCoord2f(1.0f, 1.0f);
		glVertex3f(1.0f, 1.0f, -1.0f);

		glTexCoord2f(0.0f, 1.0f);
		glVertex3f(1.0f, 1.0f, 1.0f);

		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(1.0f, -1.0f, 1.0f);
		
		// Left Face
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(-1.0f, -1.0f, -1.0f);
		
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f(-1.0f, -1.0f, 1.0f);
		
		glTexCoord2f(1.0f, 1.0f);
		glVertex3f(-1.0f, 1.0f, 1.0f);
		
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f(-1.0f, 1.0f, -1.0f);
	glEnd();
	
	glPopMatrix();
	
	xrot += 0.3f;
	yrot += 0.2f;
	zrot += 0.4f;
	
	glutSwapBuffers();
}

void keyboard (unsigned char key, int x, int y)  
{
	switch ( key ) 
	{
		case 27:
			exit (0);
			break; 
	}
}

void arrow_keys (int a_keys, int x, int y)  
{
	switch (a_keys) 
	{
		case GLUT_KEY_UP:
			glutFullScreen();
			break;
		case GLUT_KEY_DOWN:
			glutInitWindowSize(700, 500); 
			break;
	}
}

void main(int argc, char** argv)
{
	glutInit(&argc, argv); 
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA); 
	glutInitWindowPosition(0, 0);
	glutInitWindowSize(700, 500); 
	glutCreateWindow("Cubo com Textura"); 
	glutDisplayFunc(display);  
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyboard);
	glutSpecialFunc(arrow_keys);
	glutIdleFunc(display);
		
	init();
	initTexture();

	glutMainLoop ( );          
}